/*
 *  RENODeskStyle (C) 2002 Reinout Mechant  <reno@renoproductions.com>
 *
 *  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the Q Public License and the BSD license.
 *
 *  This library is free software; you should have received a copy of the
 *  Q Public License and of the BSD License along with this library;
 *  see the files COPYING.QPL and COPYING.BSD.
 *
 */

#include "renodesktrans.h"
#include <qlayout.h>
#include <kapp.h>
#include <ksimpleconfig.h>
#include <kconfig.h>
#include <klocale.h>
#include <qvbuttongroup.h>
#include <qlayout.h>
#include <qradiobutton.h>
#include <qslider.h>
#include <qlabel.h>
#include <kcolorbutton.h>
#include <kipc.h>

void RENODeskTransConfig::load()
{
	KConfig *config = KGlobal::config();
	config->setGroup("RENODeskSettings");

	int id = config->readNumEntry("TransparencyType", Custom);
	btnTransparencyGroup->setButton(id);

	opacitySlider->setValue(config->readNumEntry("TransparencyOpacity", 10));

	id = config->readNumEntry("MenuColorType", Custom);
	btnMenuColorGroup->setButton(id);

	menucolorBtn->setColor(config->readColorEntry("MenuColor", &colorGroup().mid()));

	emit changed(false);
}

void RENODeskTransConfig::save()
{
	KConfig *config = KGlobal::config();
	config->setGroup("RENODeskSettings");
	int id = btnTransparencyGroup->id(btnTransparencyGroup->selected());
	config->writeEntry("TransparencyType", id, 1, 1);

	if(id == Custom)
	{
		config->writeEntry("TransparencyOpacity", opacitySlider->value(), 1, 1);
	}

	id = btnMenuColorGroup->id(btnMenuColorGroup->selected());
	config->writeEntry("MenuColorType", id, 1, 1);

	if(id == Custom)
	{
		config->writeEntry("MenuColor", menucolorBtn->color(), 1, 1);
	}

	config->sync();

	emit changed(false);

	KIPC::sendMessageAll((KIPC::Message)MENU_CONFIG_CHANGED, 0);
}

void RENODeskTransConfig::defaults()
{
	load();
	emit changed(true);
}

void RENODeskTransConfig::slotBtnTransparencyGroupClicked(int id)
{
	customTransparencyGroup->setEnabled(id == Custom);
	emit changed(true);
}

void RENODeskTransConfig::slotBtnMenuColorGroupClicked(int id)
{
	customMenuColorGroup->setEnabled(id == Custom);
	emit changed(true);
}

RENODeskTransConfig::RENODeskTransConfig(QWidget *parent, const char *name)
	: KCModule(parent, name)
{
	QVBoxLayout *topLayout = new QVBoxLayout(this);
	QLabel *warnLbl = new QLabel(i18n("<B>Warning:</B> for RENODesk styles only!"), this);
	topLayout->addWidget(warnLbl);
	topLayout->addSpacing(16);

/************* TRANSPARENCY ************/
	btnTransparencyGroup = new QVButtonGroup( i18n( "Transparency" ), this);
	(void)new QRadioButton(i18n("No Transparency"), btnTransparencyGroup);
	(void)new QRadioButton(i18n("Custom Transparency"), btnTransparencyGroup);
	connect(btnTransparencyGroup, SIGNAL(clicked(int)), SLOT(slotBtnTransparencyGroupClicked(int)));

	customTransparencyGroup = new QGroupBox(i18n("Custom Transparency Editor"), btnTransparencyGroup);
	QGridLayout *customLayout = new QGridLayout(customTransparencyGroup, 1, 1, 10);
	QLabel *opacityLbl = new QLabel(i18n("Opacity:"), customTransparencyGroup);
	opacitySlider = new QSlider(0, 20, 1, 10, Qt::Horizontal, customTransparencyGroup);

	customLayout->addRowSpacing(0, 12);
	customLayout->addMultiCellWidget(opacityLbl, 1, 1, 0, 0);
	customLayout->addMultiCellWidget(opacitySlider, 2, 2, 0, 0);
	customLayout->setColStretch(0, 1);
	customLayout->setRowStretch(3, 1);

	topLayout->addWidget(btnTransparencyGroup);

	topLayout->addSpacing(16);

/************* MENU COLOR ************/
	btnMenuColorGroup = new QVButtonGroup( i18n( "Menu Color" ), this);
	(void)new QRadioButton(i18n("Default Color"), btnMenuColorGroup);
	(void)new QRadioButton(i18n("Custom Color"), btnMenuColorGroup);
	connect(btnMenuColorGroup, SIGNAL(clicked(int)), SLOT(slotBtnMenuColorGroupClicked(int)));

	customMenuColorGroup = new QGroupBox(i18n("Custom Color Editor"), btnMenuColorGroup);
	QGridLayout *customMenuColorLayout = new QGridLayout(customMenuColorGroup, 1, 1, 10);
	QLabel *menucolorLbl = new QLabel(i18n("Color:"), customMenuColorGroup);
	menucolorBtn = new KColorButton(customMenuColorGroup);
	customMenuColorLayout->addRowSpacing(0, 12);
	customMenuColorLayout->addMultiCellWidget(menucolorLbl, 1, 1, 0, 0);
	customMenuColorLayout->addMultiCellWidget(menucolorBtn, 2, 2, 0, 0);
	customMenuColorLayout->setColStretch(0, 1);
	customMenuColorLayout->setRowStretch(3, 1);

	topLayout->addWidget(btnMenuColorGroup);

	topLayout->addStretch(1);

	load();
	customTransparencyGroup->setEnabled(btnTransparencyGroup->id(btnTransparencyGroup->selected()) == Custom);
	customMenuColorGroup->setEnabled(btnMenuColorGroup->id(btnMenuColorGroup->selected()) == Custom);

}

RENODeskTransConfig::~RENODeskTransConfig()
{
	;
}

#include "renodesktrans.moc"
