/*
 *  RENODeskStyle (C) 2002 Reinout Mechant  <reno@renoproductions.com>
 *
 *  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the Q Public License and the BSD license.
 *
 *  This library is free software; you should have received a copy of the
 *  Q Public License and of the BSD License along with this library;
 *  see the files COPYING.QPL and COPYING.BSD.
 *
 */

#ifndef INCLUDE_MENUITEM_DEF
#define INCLUDE_MENUITEM_DEF
#endif

#include <qbitmap.h>
#include <qdrawutil.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qheader.h>
#include <qimage.h>
#include <qmenudata.h>
#include <qobjectlist.h>
#include <qpalette.h>
#include <qpointarray.h>
#include <qpopupmenu.h>
#include <qtabbar.h>
#include <qtimer.h>
#include <qwidget.h>
#include <qwidgetlist.h>
#include <qwindowsstyle.h>

#include <kapp.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdrawutil.h>
#include <kglobal.h>
#include <kglobalsettings.h>
#include <kiconloader.h>
#include <kimageeffect.h>
#include <kipc.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kpixmapio.h>
#include <krootpixmap.h>
#include <ksharedpixmap.h>
#include <ktoolbar.h>

#include <dcopclient.h>
#include <netwm.h>
#include <unistd.h>

#include <X11/X.h>
#include <X11/Xlib.h>

#include "renodeskstyle.h"

#define HORIZONTAL	(sb->orientation() == QScrollBar::Horizontal)
#define VERTICAL	!HORIZONTAL
#define MOTIF_BORDER	2
#define SLIDER_MIN	9	// wtf does this have to do with motif? -qt coder



/***********************************************
 * CLASS : TransMenuHandler                      *
 **********************************************/

TransMenuHandler::TransMenuHandler(QObject *parent)
    : QObject(parent)
{
    pixDict.setAutoDelete(true);
    connect(kapp, SIGNAL(kipcMessage(int, int)), this,
            SLOT(slotKIPCMessage(int, int)));
    reloadSettings();
}

void TransMenuHandler::reloadSettings()
{
    pixDict.clear();
    KConfig *config = KGlobal::config();
    config->setGroup("RENODeskSettings");

    type = config->readNumEntry("TransparencyType", Custom);
    menuOpacity = config->readNumEntry("TransparencyOpacity", 10);
    menucolortype = config->readNumEntry("MenuColorType", Custom);
    menumenuColor = config->readColorEntry("MenuColor", &QApplication::palette().active().mid());
}

bool TransMenuHandler::eventFilter(QObject *obj, QEvent *ev)
{
    QPopupMenu *p = (QPopupMenu *)obj;

    if(ev->type() == QEvent::Show){
        if(type == Custom){
            QApplication::syncX();
            KPixmap *pix = new KPixmap;

            if(p->testWFlags(Qt::WType_Popup)){
                QRect r(p->x(), p->y(), p->width(), p->height());
                QRect deskR = QApplication::desktop()->rect();
                if(r.right() > deskR.right() || r.bottom() > deskR.bottom()){
                    r.setBottom(deskR.bottom());
                    r.setRight(deskR.right());
                }
                *pix = QPixmap::grabWindow(qt_xrootwin(), r.x(), r.y(),
                                           r.width(), r.height());
            }
            else{ // tear off menu
                pix->resize(p->width(), p->height());
                pix->fill(Qt::black.rgb());
            }
            if(type == Custom) {
                KPixmapEffect::fade(*pix, (((float)menuOpacity)+80)*0.01, p->colorGroup().background());
            }

            pixDict.insert(p->winId(), pix);
        }
    }
    else if(ev->type() == QEvent::Hide){
        if(type == Custom){
            pixDict.remove(p->winId());
        }
    }
    return(false);
}

void TransMenuHandler::slotKIPCMessage(int id, int)
{
    if(id == MENU_CONFIG_CHANGED){

        KConfig *config = KGlobal::config();
        config->reparseConfiguration();

        QWidgetList *list = QApplication::topLevelWidgets();
        QWidgetListIt it( *list );
        QWidget *w;
        while ((w=it.current()) != 0 ){
            ++it;
            if(w->inherits("QPopupMenu")){
                w->close();
            }
        }

        reloadSettings();

    }

}


/***********************************************
 * CLASS : RENODeskStyle                           *
 **********************************************/


// Constructor
RENODeskStyle::RENODeskStyle()
		:  KStyle()
{
	menuHandler = new TransMenuHandler(this);
}

// Destructor
RENODeskStyle::~RENODeskStyle()
{
	// Empty.
}

void RENODeskStyle::polish(QApplication * app)
{
	KStyle::polish(app);
}

void RENODeskStyle::polish(QPalette & pal)
{
	KStyle::polish(pal);
}

void RENODeskStyle::unPolish(QApplication * app)
{
	KStyle::unPolish(app);
}

void RENODeskStyle::polish(QWidget * w)
{

	if (w->isTopLevel())
	{
		return;
	}

	else if (w->inherits("QMultiLineEdit"))
	{
		QFrame * f(static_cast<QFrame *>(w));
		//I don't like the WinPanel look of QMultiLineEdit


		//f->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);

		if (f && f->lineWidth() > 1)
			f->setLineWidth(1);
	}

	else if (w->inherits("QGroupBox"))
	{
		QFrame * f(static_cast<QFrame *>(w));

		if (f->frameStyle() != QFrame::NoFrame)
		{

			if ((f->frameShape() == QFrame::HLine) || (f->frameShape() == QFrame::VLine))
			{
				f->setLineWidth(1);
				f->setMidLineWidth(1);
				f->setFrameStyle(f->frameShape() | QFrame::Sunken);
			}
			else
			{
				f->setLineWidth(1);
				f->setMidLineWidth(1);
				f->setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
			}
		}
	}
}

void RENODeskStyle::unPolish(QWidget *)
{}

void RENODeskStyle::polishPopupMenu(QPopupMenu *mnu)
{
    KStyle::polishPopupMenu(mnu);
    mnu->installEventFilter(menuHandler);
}

void RENODeskStyle::drawFocusRect(QPainter * p, const QRect & r,
                              const QColorGroup & cg, const QColor * bg, bool /*atBorder*/)
{
	QRect focusRect(r);
	if (bg) p->setPen(bg->dark());
	else p->setPen(cg.background().dark());

	p->drawRect(focusRect);
}

void RENODeskStyle::drawButton(QPainter * p,
                                 int x,
                                 int y,
                                 int w,
                                 int h,
				 const QColorGroup & g,
				 bool sunken,
				 const QBrush * /*fill*/)
{
	if ( ( w <= 16) || (h <= 16))
	{
		p->save();
		p->setPen(NoPen);
		p->setBrush(g.background());
		p->drawRect(x, y, w, h);
		p->restore();
		return;
	}

	if (sunken)
	{
		p->save();
		p->setBrush(g.highlight());
		p->setPen(g.dark());
		p->drawRect(x + 1, y + 1, w - 1, h - 1);
		p->restore();
	}
	else
	{
		p->save();
		p->setPen(g.background());
		p->setBrush(NoBrush);
		p->drawLine(x, h - 1, x + 1, h - 1);
		p->drawLine(w - 1, y, w - 1, y + 1);

		p->setPen(NoPen);
		p->setBrush(g.mid());
		p->drawRect(x + 1, y + 1, w - 1, h - 1);

		p->setBrush(g.button());
		p->setPen(g.dark());
		p->drawRect(x, y, w - 1, h - 1);
		p->restore();
	}
}

QRect RENODeskStyle::buttonRect(int x, int y, int w, int h)
{
	if ((x <= 16) || (y <= 16))
		return KStyle::buttonRect(x, y, w, h);
	else
		return KStyle::buttonRect(x - 2, y - 2, w + 2, h + 2);
}

void RENODeskStyle::drawBevelButton
(QPainter * p,
 int x,
 int y,
 int w,
 int h, const QColorGroup & g, bool sunken, const QBrush * /*fill*/)
{
	if ( ( w <= 16) || (h <= 16))
	{
		p->save();
		p->setPen(NoPen);
		p->setBrush(g.background());
		p->drawRect(x, y, w, h);
		p->restore();
		return;
	}

	if (sunken)
	{
		p->save();
		p->setBrush(g.highlight());
		p->setPen(g.dark());
		p->drawRect(x + 1, y + 1, w - 1, h - 1);
		p->restore();
	}
	else
	{
		p->save();
		p->setPen(g.background());
		p->setBrush(NoBrush);
		p->drawLine(x, y + h - 1, x + 1, y + h - 1);
		p->drawLine(x + w - 1, y, x + w - 1, y + 1);

		p->setPen(g.mid());
		p->drawLine(x + 1, y + h - 1, x + w - 2, y + h - 1);
		p->drawLine(x + w - 1, y + 1, x + w - 1, y + h - 1);

		p->setBrush(g.button());
		p->setPen(g.dark());
		p->drawRect(x, y, w - 1, h - 1);
		p->restore();
	}
}

void RENODeskStyle::drawPushButton(QPushButton * b, QPainter * p)
{
	KStyle::drawPushButton(b, p);
}

void RENODeskStyle::drawPushButtonLabel(QPushButton * b, QPainter * p)
{
	KStyle::drawPushButtonLabel(b, p);
}

void RENODeskStyle::drawScrollButton(QPainter * p,
                                       int x,
                                       int y,
                                       int w,
                                       int h,
                                       const QColorGroup & g,
                                       bool /*sunken*/, const QBrush * /*fill*/)
{

	p->save();
	p->setBrush(g.background());
	p->setPen(NoPen);
	p->drawRect(x, y, w, h);
	p->restore();
}

void RENODeskStyle::drawScrollBarControls(QPainter * p,
        const QScrollBar * sb,
        int sliderStart,
        uint controls, uint activeControl)
{
	QColorGroup g = sb->colorGroup();

#ifndef QT_NO_SCROLLBAR
#define ADD_LINE_ACTIVE ( activeControl == AddLine )
#define SUB_LINE_ACTIVE ( activeControl == SubLine )

	int sliderMin, sliderMax, sliderLength, buttonDim;
	scrollBarMetrics(sb, sliderMin, sliderMax, sliderLength,
	                 buttonDim);

	if (sliderStart > sliderMax)
	{	// sanity check
		sliderStart = sliderMax;
	}

	int b = 0;
	int dimB = buttonDim;
	QRect addB;
	QRect subB;
	QRect addPageR;
	QRect subPageR;
	QRect sliderR;
	int addX, addY, subX, subY;
	int length = HORIZONTAL ? sb->width() : sb->height();
	int extent = HORIZONTAL ? sb->height() : sb->width();

	if (HORIZONTAL)
	{
		subY = addY = (extent - dimB) / 2;
		subX = b;
		addX = length - dimB - b;
	}
	else
	{
		subX = addX = (extent - dimB) / 2;
		subY = b;
		addY = length - dimB - b;
	}

	subB.setRect(subX, subY, dimB, dimB);
	addB.setRect(addX, addY, dimB, dimB);

	int sliderEnd = sliderStart + sliderLength;
	int sliderW = extent - b * 2;
	if (HORIZONTAL)
	{
		subPageR.setRect(subB.right() + 1, b,
		                 sliderStart - subB.right() - 1, sliderW);
		addPageR.setRect(sliderEnd, b, addX - sliderEnd, sliderW);
		sliderR.setRect(sliderStart, b, sliderLength, sliderW);
	}
	else
	{
		subPageR.setRect(b, subB.bottom() + 1, sliderW,
		                 sliderStart - subB.bottom() - 1);
		addPageR.setRect(b, sliderEnd, sliderW, addY - sliderEnd);
		sliderR.setRect(b, sliderStart, sliderW, sliderLength);
	}

	bool maxedOut = (sb->maxValue() == sb->minValue());
	if (controls & AddLine)
	{
		drawScrollButton(p, addB.x(), addB.y(), addB.width(),
		                 addB.height(), g, ADD_LINE_ACTIVE,
		                 &g.brush(QColorGroup::Button));

		drawArrow(p, VERTICAL ? DownArrow : RightArrow,
		          ADD_LINE_ACTIVE, addB.x() + 2, addB.y() + 2,
		          addB.width() - 4, addB.height() - 4, g,
		          !maxedOut);
	}

	if (controls & SubLine)
	{
		drawScrollButton(p, subB.x(), subB.y(), subB.width(),
		                 subB.height(), g, SUB_LINE_ACTIVE,
		                 &g.brush(QColorGroup::Button));

		drawArrow(p, VERTICAL ? UpArrow : LeftArrow,
		          SUB_LINE_ACTIVE, subB.x() + 2, subB.y() + 2,
		          subB.width() - 4, subB.height() - 4, g,
		          !maxedOut);
	}

	QBrush br = g.background();
	p->setBrush(br);
	p->setPen(NoPen);
	p->setBackgroundMode(OpaqueMode);

	if (maxedOut)
	{
		p->drawRect(sliderR);
	}
	else
	{
		if ((controls & SubPage && SubPage == activeControl) ||
		        (controls & AddPage && AddPage == activeControl))
		{
			QBrush b = p->brush();
			QColor c = p->backgroundColor();
			p->setBackgroundColor(g.dark());

			p->setBrush(g.shadow());
			p->drawRect(AddPage ==
			            activeControl ? addPageR : subPageR);
			p->setBackgroundColor(c);
			p->setBrush(b);
		}
		if (controls & SubPage && SubPage != activeControl)
			p->drawRect(subPageR);
		if (controls & AddPage && AddPage != activeControl)
			p->drawRect(addPageR);
		if (controls & Slider)
		{
			QPoint bo = p->brushOrigin();
			p->setBrushOrigin(sliderR.topLeft());

			if (VERTICAL)
			{
				p->setPen(g.dark());
				p->setBrush(g.background().light(110));
				p->drawRect(sliderR.x() + 2, sliderR.y(), sliderR.width() - 4, sliderR.height() - 1);

				p->setPen(g.mid());
				p->setBrush(NoBrush);

				//draw horizontal line across bottom
				p->drawLine(sliderR.x() + 2, sliderR.y() + sliderR.height() - 1, sliderR.x() + sliderR.width() -2, sliderR.y() + sliderR.height() - 1);
				//draw vertical line along right
				p->drawLine(sliderR.x() + sliderR.width() - 2, sliderR.y(), sliderR.x() + sliderR.width() - 2, sliderR.y() + sliderR.height() - 1);

				//draw vertical lines along left and right to cover background
				p->setPen(g.background());
				p->drawLine(sliderR.x() + sliderR.width() - 1, sliderR.y(), sliderR.x() + sliderR.width() - 1, sliderR.y() + sliderR.height() - 1);
				p->drawLine(sliderR.x() + 1, sliderR.y(), sliderR.x() + 1, sliderR.y() + sliderR.height() - 1);
			}
			else //horizontal slider
			{
				p->setPen(g.dark());
				p->setBrush(g.background().light(110));
				p->drawRect(sliderR.x(), sliderR.y() + 2, sliderR.width() - 1, sliderR.height() - 4);

				p->setPen(g.mid());
				p->setBrush(NoBrush);

				//draw vertical line along right
				p->drawLine(sliderR.x() + sliderR.width() - 1, sliderR.y() + 2, sliderR.x() + sliderR.width() - 1, sliderR.y() + sliderR.height() - 2);

				//draw horizontal line across bottom
				p->drawLine(sliderR.x(), sliderR.y() + sliderR.height() - 2, sliderR.x() + sliderR.width() - 1, sliderR.y() + sliderR.height() - 2);
				//draw horizontal lines along top and bottom to fill background
				p->setPen(g.background());
				p->drawLine(sliderR.x(), sliderR.y() + sliderR.height() - 1, sliderR.x() + sliderR.width() - 1, sliderR.y() + sliderR.height() - 1);
				p->drawLine(sliderR.x(), sliderR.y() + 1, sliderR.x() + sliderR.width() - 1, sliderR.y() + 1);
			}

			p->setBrushOrigin(bo);
		}
	}

	//now draw a line separating scrollbar from content are
	p->setPen(g.dark());
	p->setBrush(NoBrush);

	if (VERTICAL)
	{
		p->drawLine(subX, subY, subX, length);
	}
	else
	{
		p->drawLine(subX, subY, length, subY);
	}


	// ### perhaps this should not be able to accept focus if maxedOut?
	if (sb->hasFocus() && (controls & Slider))
		drawFocusRect(p, QRect(sliderR.x() + 2, sliderR.y() + 2,
		                       sliderR.width() - 5,
		                       sliderR.height() - 5), g,
		              &sb->backgroundColor());
#endif
}

QStyle::ScrollControl
RENODeskStyle::scrollBarPointOver(const QScrollBar * sb, int sliderStart,
                                    const QPoint & p)
{
	if (!sb->rect().contains(p))
		return (NoScroll);

	int sliderMin, sliderMax, sliderLength, buttonDim;
	scrollBarMetrics(sb, sliderMin, sliderMax, sliderLength,
	                 buttonDim);

	if (sb->orientation() == QScrollBar::Horizontal)
	{
		int x = p.x();
		if (x < buttonDim)
			return (SubLine);
		else if (x < sliderStart)
			return (SubPage);
		else if (x < sliderStart + sliderLength)
			return (Slider);
		else if (x < sliderMax + sliderLength)
			return (AddPage);

		return (AddLine);
	}
	else
	{
		int y = p.y();
		if (y < buttonDim)
			return (SubLine);
		else if (y < sliderStart)
			return (SubPage);
		else if (y < sliderStart + sliderLength)
			return (Slider);
		else if (y < sliderMax + sliderLength)
			return (AddPage);

		return (AddLine);
	}
}

void RENODeskStyle::scrollBarMetrics(const QScrollBar * sb, int &sliderMin,
                                       int &sliderMax, int &sliderLength,
                                       int &buttonDim)
{
	int maxLength;
	bool horizontal = sb->orientation() == QScrollBar::Horizontal;
	int len = (horizontal) ? sb->width() : sb->height();
	int extent = (horizontal) ? sb->height() : sb->width();
	int b = 0;		// border around sbar

	if (len > (extent - 1) * 2)
		buttonDim = extent - b * 2;
	else
		buttonDim = (len - b * 2) / 2 - 1;

	sliderMin = buttonDim + b;
	maxLength = len - b * 2 - buttonDim * 2;

	if (sb->maxValue() == sb->minValue())
	{
		sliderLength = maxLength;
	}
	else
	{
		sliderLength = (sb->pageStep() * maxLength) /
		               (sb->maxValue() - sb->minValue() + sb->pageStep());
		uint range = sb->maxValue() - sb->minValue();
		if (sliderLength < 22 || range > INT_MAX / 2)
			sliderLength = 22;
		if (sliderLength > maxLength)
			sliderLength = maxLength;
	}

	sliderMax = sliderMin + maxLength - sliderLength;
}

QSize RENODeskStyle::indicatorSize() const
{
	return KStyle::indicatorSize();
}

//draw a check box
void RENODeskStyle::drawIndicator(QPainter * p,
                                    int x,
                                    int y,
                                    int w,
                                    int h, const QColorGroup & g, int state,
                                    bool down, bool enabled)
{
	int l;
	p->save();

	if (w != h)
	{
		l = (w < h ? w : h);
	}
	else
	{
		l = h = w;
	}
	if (l % 2)
	{
		--l;
	}


	const int il = l - 6;
	const int ix = x + ((w - (il)) / 2);
	const int iy = y + ((h - (il)) / 2);

	const int ol = l - 4;
	const int ox = x + ((w - (ol)) / 2);
	const int oy = y + ((h - (ol)) / 2);

	if (enabled)
	{
		if (down)
			p->setPen(g.highlight());
		else
			p->setPen(g.dark());
	}
	else
	{
		p->setPen(g.background());
	}

	p->fillRect(x, y, w, h, g.background());
	p->fillRect(ox + 1, oy + 1, ol, ol, g.mid());

	p->setBrush(g.background());
	p->drawRect(ox, oy, ol, ol);

	if (state != QButton::Off)
	{
		p->fillRect(ix, iy, il, il,
		            enabled ? g.highlight() : g.background());
		if (state == QButton::NoChange)
		{
			p->fillRect(ix, iy, il, il, g.background());
		}
	}

	p->restore();
}

QSize RENODeskStyle::exclusiveIndicatorSize() const
{
	return KStyle::exclusiveIndicatorSize();
}

//draw a radio button
void RENODeskStyle::drawExclusiveIndicator
(QPainter * p,
 int x,
 int y,
 int w,
 int h, const QColorGroup & g, bool on, bool down, bool enabled)
{
	p->save();
	p->fillRect(x, y, w, h, g.background());

	w -= 1;
	h -= 1;

	// Avoid misshapen ellipses. Qt or X bug ? Who knows...
	if (0 == w % 2)
		--w;

	if (0 == h % 2)
		--h;

	p->setPen(NoPen);
	p->setBrush(g.mid());
	p->drawEllipse(x + 2, y + 2, w, h);

	if (enabled)
	{
		p->setPen(down ? g.highlight() : g.dark());
	}
	else
	{
		p->setPen(g.mid());
	}

	x++;
	y++;
	p->setBrush(g.background());
	p->drawEllipse(x, y, w, h);

	if (on)
	{
		p->setPen(enabled ? g.highlight() : g.mid());
		p->setBrush(enabled ? g.highlight() : g.mid());
		p->drawEllipse(x + 3, y + 3, w - 6, h - 6);
	}

	p->restore();
}

void RENODeskStyle::drawIndicatorMask
(QPainter * p, int x, int y, int w, int h, int)
{
	int l;
	(w != h ? (l = (w < h ? w : h)) : (l = h = w));

	if (l % 2)
	{
		--l;
	}

	p->fillRect(x + ((w - (l - 4)) / 2) + 1, y + ((h - (l - 4)) / 2),
	            l - 4, l - 4, Qt::color1);
}

void RENODeskStyle::drawExclusiveIndicatorMask
(QPainter * p, int x, int y, int w, int h, bool)
{
	if (0 == w % 2)
		--w;

	if (0 == h % 2)
		--h;

	p->setPen(Qt::color1);
	p->setBrush(Qt::color1);
	p->drawEllipse(x + 1, y, w, h);
}

void RENODeskStyle::drawComboButton
(QPainter * p,
 int x,
 int y,
 int w,
 int h,
 const QColorGroup & g, bool /*sunken*/,
 bool, bool, const QBrush *)
{
	p->save();

	static const unsigned char downarrow_bits[] =
	    {
	        0x7f, 0xbe, 0x9c, 0x08, 0x00, 0x00, 0x00, 0x28,
	        0x00, 0x00, 0x00, 0x11, 0x00, 0x00, 0x00, 0xb8
	    };

	static const unsigned int handle_width = 13;
	static const unsigned int handle_offset = handle_width;

	QBitmap downArrow = QBitmap(7, 4, downarrow_bits, true);
	downArrow.setMask(downArrow);

	//fill pixel on right top and left bottom with bg
	p->setPen(g.background());
	p->drawLine(x + w - handle_offset - 2, y, x + w - handle_offset, y);
	p->drawLine(x, y + h - 1, x + 1, y + h - 1);

	//draw shadows
	p->setPen(g.mid());
	//draw vertical shadow line on right
	p->drawLine(x + w - handle_offset - 1, y + 1, x + w - handle_offset - 1, y + h);
	//draw horizontal shadow line along bottom
	p->drawLine(x + 1, y + h - 1, x + w - handle_offset, y + h - 1);

	p->setPen(g.dark());
	p->setBrush(g.base());
	p->drawRect(x, y, w - handle_offset - 1, h - 1);

	p->fillRect(w - handle_offset, y, handle_width, h, g.background());

	p->setBrush(g.dark());

	QRect cr(w - handle_offset, y + 2, handle_width, h - 4);
	QRect pmr(0, 0, 7, 4);

	p->setPen(g.foreground());
	pmr.moveCenter(cr.center());
	p->drawPixmap(pmr.topLeft(), downArrow);

	p->restore();
}

QRect RENODeskStyle::comboButtonRect(int x, int y, int w, int h)
{
	return KStyle::comboButtonRect(x, y, w, h);
}

QRect RENODeskStyle::comboButtonFocusRect(int x, int y, int w, int h)
{
	return KStyle::comboButtonFocusRect(x, y, w, h);
}

int RENODeskStyle::sliderLength() const
{
	return KStyle::sliderLength();
}

void RENODeskStyle::drawSliderGroove
(QPainter * p,
 int x,
 int y, int w, int h, const QColorGroup & g, QCOORD /*c*/, Orientation o)
{
	p->save();

	p->fillRect(x, y, w, h, g.background());

	p->setPen(NoPen);
	p->setBrush(g.mid().dark(120));
	if (o == Qt::Horizontal)
	{
		int bTop = y + ( h / 2) - 2;

		for (int k = 0; k < w; k += 3)
		{
			p->drawRect(x + k, bTop, 1, 3);
		}
	}

	else if (o == Qt::Vertical)
	{
		int bLeft = x + ( w / 2) - 2;

		for (int k = 0; k < h; k += 3)
		{
			p->drawRect(bLeft, y + k, 3, 1);
		}
	}

	p->restore();
}

void RENODeskStyle::drawArrow
(QPainter * p,
 Qt::ArrowType type,
 bool down,
 int x,
 int y,
 int w,
 int h, const QColorGroup & g, bool enabled, const QBrush * fill)
{
	KStyle::drawArrow(p, type, down, x, y, w, h, g, enabled, fill);
}

void RENODeskStyle::drawSlider
(QPainter * p,
 int x,
 int y,
 int w,
 int h,
 const QColorGroup & g,
 Orientation o, bool /*tickAbove*/, bool /*tickBelow*/)
{
	p->save();
	p->fillRect(x, y, w, h, g.background());


	int sl = sliderLength() - 1;
	if (0 == (sl % 2)) sl--;

	if( o == Qt::Horizontal )
	{
		p->setPen(NoPen);
		p->setBrush(g.mid());
		p->drawEllipse(x + 1, (y + h / 2 - sl / 2) + 1, sl, sl);

		p->setPen(g.dark());
		p->setBrush(g.light());
		p->drawEllipse(x, (y + h / 2 - sl / 2), sl, sl);
	}
	else if( o == Qt::Vertical )
	{
		p->setPen(NoPen);
		p->setBrush(g.mid());
		p->drawEllipse((x + w / 2 - sl / 2) + 1, y + 1, sl, sl);

		p->setPen(g.dark());
		p->setBrush(g.light());
		p->drawEllipse((x + w / 2 - sl / 2), y, sl, sl);
	}

	p->restore();
}

void RENODeskStyle::drawKToolBar
(QPainter * p,
 int x,
 int y,
 int w, int h, const QColorGroup & /*g*/, KToolBarPos, QBrush * /*fill*/)
{
	p->fillRect(x, y, w, h, QApplication::palette().active().background());
}

void RENODeskStyle::drawKBarHandle
(QPainter * p,
 int x,
 int y,
 int w, int h, const QColorGroup & g, KToolBarPos pos,
 QBrush * /*fill*/)
{
	int k;
	p->save();

	//Use QApp palette because @fill uses button
	//background color... which doesn't look good here
	p->setBrush(QApplication::palette().active().background());
	p->setPen(NoPen);
	p->drawRect(x, y, w, h);
	p->setBrush(g.mid());
	if(pos == 2 || pos == 3)
	{
		for (k = 4; k < h - 4; k += 3)
		{
			p->drawRect(x + 2, k, w - 4, 1);
		}
	}
	else
	{
		for (k = 4; k < w - 4; k += 3)
		{
			p->drawRect(k, y, 1, h - 4);
		}
	}

	p->restore();
}

void RENODeskStyle::drawKMenuBar
(QPainter * p,
 int x, int y, int w, int h, const QColorGroup & g, bool macmode, QBrush *fill)
{
	QBrush bgFill = (fill ? *fill : g.brush(QColorGroup::Background));
	p->fillRect(x, y, w, h, bgFill);

	if (macmode)
	{
		p->setPen(g.mid());
		p->drawLine(x, y + h - 1, x + w, y + h - 1);
	}
}

void RENODeskStyle::drawKToolBarButton(QPainter *p, int x, int y, int w, int h,
                                    const QColorGroup &g, bool sunken,
                                    bool raised, bool enabled, bool popup,
                                    KToolButtonType icontext,
                                    const QString& btext, const QPixmap *pixmap,
                                    QFont *font, QWidget *)
{
    int dx, dy;

    QFontMetrics fm(*font);

    if(raised || sunken)
    {
        drawBevelButton(p, x, y, w, h, g, sunken, NULL);
    }
    else
    {
        p->fillRect(x, y, w, h, QApplication::palette().active().background());
    }

    p->setPen(g.buttonText());

    if (icontext == Icon){ // icon only
        if (pixmap){
            dx = ( w - pixmap->width() ) / 2;
            dy = ( h - pixmap->height() ) / 2;
            if ( sunken )
            {
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
    }
    else if (icontext == IconTextRight){ // icon and text (if any)
        if (pixmap){
            dx = 4;
            dy = ( h - pixmap->height() ) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
        if (!btext.isNull()){
            int tf = AlignVCenter|AlignLeft;
            if (pixmap)
                dx= 4 + pixmap->width() + 2;
            else
                dx= 4;
            dy = 0;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x+dx, y+dy, w-dx, h, tf, btext);
        }
    }
    else if (icontext == Text){ // only text, even if there is a icon
        if (!btext.isNull()){
            int tf = AlignVCenter|AlignLeft;
            if (!enabled)
                p->setPen(g.dark());
            dx= (w - fm.width(btext)) / 2;
            dy= (h - fm.lineSpacing()) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x+dx, y+dy, fm.width(btext), fm.lineSpacing(), tf, btext);
        }
    }
    else if (icontext == IconTextBottom){
        if (pixmap){
            dx = (w - pixmap->width()) / 2;
            dy = (h - fm.lineSpacing() - pixmap->height()) / 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            p->drawPixmap( x+dx, y+dy, *pixmap );
        }
        if (!btext.isNull()){
            int tf = AlignBottom|AlignHCenter;
            dy= pixmap->height();
            dx = 2;
            if ( sunken ){
                ++dx;
                ++dy;
            }
            if (font)
                p->setFont(*font);
            if(raised)
                p->setPen(KGlobalSettings::toolBarHighlightColor());
            p->drawText(x, y, w, h-3, tf, btext);
        }
    }
    if (popup){
        if (enabled)
            qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle, false, w-5, h-5, 0, 0,
                        g, true);
        else
            qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle, false, w-5, h-5,
                        0, 0, g, false);
    }
}


void RENODeskStyle::drawKMenuItem(QPainter * p,
                                    int x,
                                    int y,
                                    int w,
                                    int h,
                                    const QColorGroup & g,
                                    bool active, QMenuItem * mi, QBrush *)
{
	if (p->font() == KGlobalSettings::generalFont())
		p->setFont(KGlobalSettings::menuFont());

	if (active)
	{
		p->setPen(g.highlight().dark());
		p->drawRect(x, y, w, h);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.highlight());

		QString s = mi->text();
		int t = s.find('\t');
		const int text_flags =
		    AlignVCenter | ShowPrefix | DontClip | SingleLine;
		p->setPen(g.highlightedText());
		p->drawText(x + 5, y + 3, w - 10, h - 6, text_flags, s, t);
	}
	else
	{
		QApplication::style().drawItem(p, x, y, w, h,
		                               AlignCenter | ShowPrefix |
		                               DontClip | SingleLine, g,
		                               mi->isEnabled(),
		                               mi->pixmap(), mi->text(),
		                               -1, &g.buttonText());
	}
}

void RENODeskStyle::drawPopupMenuItem
(QPainter * p,
 bool /*checkable*/,
 int maxpmw,
 int tab,
 QMenuItem * mi,
 const QPalette & pal,
 bool act, bool enabled, int x, int y, int w, int h)
{
	static const int motifItemFrame = 0;
	static const int motifItemHMargin = 7;
	static const int motifItemVMargin = 7;
	static const int motifArrowHMargin = 6;
	static const int windowsRightBorder = 12;

	bool dis = !enabled;
	QColorGroup itemg = dis ? pal.disabled() : pal.active();

	maxpmw = QMAX(maxpmw, 20);
	int checkcol = maxpmw;

	if (p->font() == KGlobalSettings::generalFont())
		p->setFont(KGlobalSettings::menuFont());

	if (!mi)
	{
		return;
	}

	if (mi->isSeparator())
	{
		if (menuHandler->transType() == none) {
			p->fillRect(x, y, w, h, itemg.background());
		} else {
			KPixmap *pix = menuHandler->pixmap(((QWidget*)p->device())->winId());
			if (pix)
				p->drawPixmap(x, y, *pix, x, y, w, h);
		}
		p->setPen(itemg.mid());
		p->drawLine(x + 28, y + 1, x + w, y + 1);
		if(menuHandler->menucolorType() == Custom) {
			p->fillRect(x, y, 22, h, menuHandler->menuColor());
		} else {
			p->fillRect(x, y, 22, h, itemg.mid());
		}
		return;
	}

	if (act && enabled)
	{
		// draw highlight around current menu item
		QPen oldPen = p->pen();

		p->setPen(itemg.highlight().dark());
		p->setBrush(itemg.highlight());
		p->drawRect(x, y, w, h);

		p->setPen(oldPen);
	}
	else
	{
		if (menuHandler->transType() == none) {
			p->fillRect(x, y, w, h, itemg.background());
		} else {
			KPixmap *pix = menuHandler->pixmap(((QWidget*)p->device())->winId());
			if (pix)
				p->drawPixmap(x, y, *pix, x, y, w, h);
		}
		if(menuHandler->menucolorType() == Custom) {
			p->fillRect(x, y, 22, h, menuHandler->menuColor());
		} else {
			p->fillRect(x, y, 22, h, itemg.mid());
		}
	}
	if (mi->isChecked() && !mi->iconSet())
	{
		// draw check mark
		int w2 = checkcol - 2;
		int h2 = h - 2;

		const int ol = 10;
		const int ox = x + 2 + ((w2 - ol) / 2);
		const int oy = y + 1 + ((h2 - ol) / 2);
		if (w2 < 10 || h2 < 10)
			return;

		p->save();
		p->setPen(itemg.dark());
		mi->isChecked() ? p->setBrush(itemg.highlight()) :
		p->setBrush(itemg.background());
		p->drawRect(ox, oy, ol, ol);
		p->restore();
		//      drawIndicator(p, x + 2, y + 1, w2, h2, itemg, TRUE, FALSE,
		//                    enabled);
	}
	else if (mi->iconSet())
	{
		// draw associated icon
		QIconSet::Mode mode =
		    dis ? QIconSet::Disabled : QIconSet::Normal;
		if (!dis)
			mode = QIconSet::Active;
		QPixmap pixmap =
		    mi->iconSet()->pixmap(QIconSet::Small, mode);
		int pixw = pixmap.width();
		int pixh = pixmap.height();

		QRect cr(x, y, checkcol, h);
		QRect pmr(0, 0, pixw, pixh);
		pmr.moveCenter(cr.center());
		p->setPen(itemg.highlightedText());
		p->drawPixmap(pmr.topLeft(), pixmap);
	}

	p->setPen(itemg.buttonText());
	QColor discol;
	if (dis)
	{
		discol = itemg.text();
		p->setPen(discol);
	}

	int xm = motifItemFrame + checkcol + motifItemHMargin;

	QString s = mi->text();
	if (!s.isNull())
	{
		int t = s.find('\t');
		int m = motifItemVMargin;
		const int text_flags =
		    AlignVCenter | ShowPrefix | DontClip | SingleLine;
		QPen oldPen = p->pen();
                p->setPen(oldPen);
		if (t >= 0)
		{
			if (dis)
			{
				p->setPen(itemg.highlightedText());
				p->drawText(x + w - tab -
				            windowsRightBorder -
				            motifItemHMargin -
				            motifItemFrame + 1, y + m + 1,
				            tab, h - 2 * m, text_flags,
				            s.mid(t + 1));
				p->setPen(discol);
			}
			if (act && enabled)
			{
				p->setPen(itemg.highlightedText());
			}
			p->drawText(x + w - tab - windowsRightBorder -
			            motifItemHMargin - motifItemFrame,
			            y + m, tab, h - 2 * m, text_flags,
			            s.mid(t + 1));
		}

		if (dis)
		{
			p->setPen(itemg.highlightedText());
			p->drawText(x + xm + 1, y + m + 1, w - xm + 1,
			            h - 2 * m, text_flags, s, t);
			p->setPen(discol);
		}
		if (act && enabled)
		{
			p->setPen(itemg.highlightedText());
		}
		p->drawText(x + xm, y + m, w - xm - tab + 1, h - 2 * m,
		            text_flags, s, t);
	}
	else if (mi->pixmap())
	{
		QPixmap *pixmap = mi->pixmap();
		if (pixmap->depth() == 1)
			p->setBackgroundMode(OpaqueMode);
		p->drawPixmap(x + xm, y + motifItemFrame, *pixmap);
		if (pixmap->depth() == 1)
			p->setBackgroundMode(TransparentMode);
	}

	if (mi->popup())
	{
		int dim = (h - 2 * motifItemFrame) / 2;
		if (act)
		{
			if (!dis)
				discol = itemg.buttonText();

			QColorGroup itemg2(discol, itemg.highlight(),
			                   white, white,
			                   dis ? discol : white,
			                   discol, white);
			drawArrow(p, RightArrow, FALSE,
			          x + w - motifArrowHMargin -
			          motifItemFrame - dim,
			          y + h / 2 - dim / 2, dim, dim, itemg2,
			          TRUE);
		}
		else
		{
			drawArrow(p, RightArrow, FALSE,
			          x + w - motifArrowHMargin -
			          motifItemFrame - dim,
			          y + h / 2 - dim / 2, dim, dim, itemg,
			          mi->isEnabled());

		}
	}
	//  KStyle::drawPopupMenuItem(p, checkable, maxpmw, tab, mi, pal, act, enabled, x, y, w, h);
}

int RENODeskStyle::popupMenuItemHeight(bool, QMenuItem * mi,
        const QFontMetrics & fm)
{
	if (mi->isSeparator())
		return 3;

	int h = 0;
	if (mi->pixmap())
		h = mi->pixmap()->height();

	if (mi->iconSet())
		h = QMAX(mi->iconSet()->
		         pixmap(QIconSet::Small,
		                QIconSet::Normal).height(), h);

	h = QMAX(fm.height() + 6, h);

	// we want a minimum size of 16
	h = QMAX(h, 16);

	return h;
}

void RENODeskStyle::drawKProgressBlock
(QPainter * p,
 int x, int y, int w, int h, const QColorGroup & g, QBrush * fill)
{
	KStyle::drawKProgressBlock(p, x, y, w, h, g, fill);
}

void RENODeskStyle::drawPanel
(QPainter * p,
 int x,
 int y,
 int w,
 int h,
 const QColorGroup & g,
 bool sunken, int /* lineWidth */, const QBrush *brush)
{
	p->save();

	QBrush fill = (brush ? *brush : NoBrush);

	/*
		I don't want to use the same routines to draw
		The KDE Panel as normal panels. Trouble is, applets
		running in the Kicker will not be able to draw QFrames
		properly. This stinks. All hail Qt 3.0, which won't have
		such a nasty theming API.
	*/
		//if would seem that some panels which *should* be drawn as VLines or HLines
		//manage to squeeze by... but this doesn't seem to fix it.
		if (( w < 6) || (h < 6))
		{
			drawSeparator(p, x, y, w, h, g, true, 1, 1);
		}
		else
		{

			//I know these are look the same... but sunken panels are inset on left and top by 1 pixel
			if (sunken)
			{
				p->setPen(g.background());
				p->setBrush(NoBrush);
				p->drawLine(x, h - 1, x + 1, h - 1);
				p->drawLine(w - 1, y, w - 1, y + 1);

				p->setPen(g.dark());
				p->setBrush(fill);
				p->drawRect(x + 1, y + 1, w - 2, h - 2);

				p->setPen(g.mid());
				p->setBrush(NoBrush);
				//draw horizontal shadow line along bottom
				p->drawLine(x + 2, y + h - 1, x + w, y + h - 1);
				//draw vertical shadow line across right
				p->drawLine(x + w - 1, y + 2, x + w - 1, y + h);

			}
			else
			{
				p->setPen(g.background());
				p->setBrush(NoBrush);
				p->drawLine(x, h - 1, x + 1, h - 1);
				p->drawLine(w - 1, y, w - 1, y + 1);

				p->setPen(g.dark());
				p->setBrush(fill);
				p->drawRect(x, y, w - 1, h - 1);

				p->setPen(g.mid());
				p->setBrush(NoBrush);
				//draw horizontal shadow line along bottom
				p->drawLine(x + 1, y + h - 1, x + w, y + h - 1);
				//draw vertical shadow line across right
				p->drawLine(x + w - 1, y + 1, x + w - 1, y + h);
			}
		}

	p->restore();
}

void RENODeskStyle::drawPopupPanel
(QPainter * p,
 int x,
 int y, int w, int h, const QColorGroup & g, int, const QBrush *)
{
	p->setPen(g.dark());
	p->setBrush(g.background());
	p->drawRect(x, y, w, h);
	if(menuHandler->menucolorType() == Custom) {
		p->fillRect(x+1, y+1, 23, h-2, menuHandler->menuColor());
	} else {
		p->fillRect(x+1, y+1, 23, h-2, g.mid());
	}
}

void RENODeskStyle::drawSeparator
(QPainter * p,
 int x,
 int y,
 int w,
 int h,
 const QColorGroup & g, bool /*sunken*/, int /*lineWidth*/, int /*midLineWidth*/)
{
	if (w > h) //horizontal
	{
		p->fillRect(x, y, w, h, g.background());

		int yMid = h / 2;
		p->setPen(g.mid());

		p->drawLine(x + 4, y + yMid, x + w - 4, y + yMid);
	}
	else //vertical
	{
		p->fillRect(x, y, w, h, g.background());

		int xMid = w / 2;
		p->setPen(g.mid());
		p->drawLine(x + xMid, y + 4, x + xMid, y + h - 4);
	}
}

void RENODeskStyle::drawTab
(QPainter * p, const QTabBar * tabBar, QTab * tab, bool selected)
{
	p->save();

	QRect r(tab->rect());
	QColorGroup g(tabBar->colorGroup());

	//Default, there's a missing pixel to the lower right of the first
	//tab, for reasons I'm not 100% certain of. So we'll kludge it :P
	bool firstTab = (tab->id == 0);
	bool lastTab = (tab->id == (tabBar->count() - 1));

	if (firstTab)
	{
		p->setPen(g.dark());
		p->drawLine(r.left(), r.bottom(), r.left(), r.bottom() + 1);
	}


	if ((tabBar->shape() == QTabBar::RoundedAbove) || (tabBar->shape() == QTabBar::TriangularAbove))
	{
		r.setBottom(r.bottom() - 1); //take off one pixel from bottom
	}
	else
	{
		r.setTop(r.top() + 1);
	}

	switch (tabBar->shape())
	{
	case QTabBar::RoundedAbove:
	case QTabBar::TriangularAbove:


		if (selected)
		{
			//fill tab
			r.setRight(r.right() - 1);
			p->fillRect(r, g.background());
			p->setPen(g.dark());

			//draw border (but not bottom)
			p->drawLine(r.left(), r.top(), r.left(), r.bottom());
			p->drawLine(r.left(), r.top(), r.right(), r.top());
			p->drawLine(r.right(), r.top(), r.right(), r.bottom());

			//draw shadow line
			p->setPen(g.mid().dark(110));
			p->drawLine(r.right() + 1, r.top() + 1, r.right() + 1, r.bottom() - 1);

			if (lastTab)
			{
				p->setPen(g.dark());
				p->drawLine(r.right(), r.bottom(), r.right() + 1, r.bottom());
			}
		}
		else
		{
			r.setTop(r.top() + 2);

			p->fillRect(r, g.background().dark(105));
			p->setPen(g.mid());

			p->drawLine(r.left(), r.top(), r.left(), r.bottom());
			p->drawLine(r.left(), r.top(), r.right(), r.top());
			p->drawLine(r.right(), r.top(), r.right(), r.bottom());

			p->setPen(g.dark());
			p->drawLine(r.left(), r.bottom(), r.right(), r.bottom());
		}

		break;
	case QTabBar::RoundedBelow:
	case QTabBar::TriangularBelow:

		if (selected)
		{
			//fill tab
			r.setRight(r.right() - 1);
			p->fillRect(r, g.background());
			p->setPen(g.dark());

			//draw border (but not bottom)
			p->drawLine(r.left(), r.top(), r.left(), r.bottom());
			p->drawLine(r.left(), r.top(), r.right(), r.top());
			p->drawLine(r.right(), r.top(), r.right(), r.bottom());

			//draw shadow line
			p->setPen(g.mid().dark(110));
			p->drawLine(r.right() + 1, r.top() + 1, r.right() + 1, r.bottom() - 1);

			if (lastTab)
			{
				p->setPen(g.dark());
				p->drawLine(r.right(), r.bottom(), r.right() + 1, r.bottom());
			}
		}
		else
		{
			r.setTop(r.top() + 2);

			p->fillRect(r, g.background().dark(105));
			p->setPen(g.mid());

			p->drawLine(r.left(), r.top(), r.left(), r.bottom());
			p->drawLine(r.left(), r.top(), r.right(), r.top());
			p->drawLine(r.right(), r.top(), r.right(), r.bottom());

			p->setPen(g.dark());
			p->drawLine(r.left(), r.bottom(), r.right(), r.bottom());
		}

		break;
	}

	p->restore();
}

void RENODeskStyle::drawTabMask
(QPainter * p, const QTabBar * /*tabBar*/, QTab * tab, bool /*selected*/)
{
	p->fillRect(tab->rect(), Qt::color1);
}


void RENODeskStyle::drawKickerHandle
(QPainter * p,
 int x, int y, int w, int h, const QColorGroup & g, QBrush *)
{
	int k;
	p->save();
	p->setPen(NoPen);
	p->fillRect(x, y, w, h, QApplication::palette().active().background());
	if (w < h)
		//	Must be a vertical handle.
		for (k = 4; k < h - 4; k += 3)
		{
			p->fillRect(x + 2, k, w - 4, 1, g.mid());
		}
	else
		//	Must be a horizontal handle.
		for (k = 4; k < w - 4; k += 3)
		{
			p->fillRect(k, y + 2, 1, h - 4, g.mid());
		}
	p->restore();
}

void RENODeskStyle::drawKickerAppletHandle
(QPainter * p,
 int x, int y, int w, int h, const QColorGroup & g, QBrush *)
{
	int k;
	p->save();
	p->setPen(NoPen);
	p->fillRect(x, y, w, h, QApplication::palette().active().background());
	if (w < h)
		//	Must be a vertical handle.
		for (k = 4; k < h - 4; k += 3)
		{
			p->fillRect(x + 2, k, w - 4, 1, g.mid());
		}
	else
		//	Must be a horizontal handle.
		for (k = 4; k < w - 4; k += 3)
		{
			p->fillRect(k, y + 2, 1, h - 4, g.mid());
		}
	p->restore();
	//  KStyle::drawKickerAppletHandle(p, x, y, w, h, g, brush);
}

void RENODeskStyle::drawKickerTaskButton
(QPainter * p,
 int x,
 int y,
 int w,
 int h,
 const QColorGroup & g,
 const QString &, bool active, QPixmap *, QBrush *)
{
	p->save();
	p->setPen(g.mid());
	p->drawRect(x, y, w, h);
	p->setPen(g.midlight());
	if (active)
	{
		p->drawLine(w, h, x, h);
		p->drawLine(w, h, w, y);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.background());
	}
	else
	{
		p->drawLine(x, y, x, h);
		p->drawLine(x, y, w, y);
		p->fillRect(x + 1, y + 1, w - 2, h - 2, g.background());
	}
	p->restore();
}

bool RENODeskStyle::eventFilter(QObject * obj, QEvent * ev)
{
	//I don't like highlighting of widgets on mouseover... but I'm leaving the
	//code in place, in case I change my mind


	if ( obj->inherits("QButton") || obj->inherits("QComboBox"))
	{
		if (ev->type() == QEvent::Enter)
		{
			QWidget *btn = (QWidget *) obj;
			if (btn->isEnabled())
			{
				QPalette pal = btn->palette();
				pal.setColor(QColorGroup::Mid,
				             pal.active().
				             color(QColorGroup::
				                   Highlight));
				pal.setColor(QColorGroup::Shadow,
				             pal.active().
				             color(QColorGroup::Highlight).
				             dark());
				btn->setPalette(pal);
			}
		}
		else if (ev->type() == QEvent::Leave)
		{
			QWidget *btn = (QWidget *) obj;
			QPalette pal = btn->palette();
			pal.setColor(QColorGroup::Mid,
			             pal.active().
			             color(QColorGroup::Background));
			pal.setColor(QColorGroup::Shadow,
			             pal.active().
			             color(QColorGroup::Light));
			btn->setPalette(pal);
		}
	}


	return false;
}

void RENODeskStyle::drawSplitter ( QPainter *p, int x, int y, int w, int h,
		    const QColorGroup & g, Orientation o)
{
	p->save();
	p->setPen(NoPen);
	p->setBrush(g.background());
	p->drawRect(x, y, w, h);

	p->setPen(NoPen);
	p->setBrush(g.mid().dark(120));

	int grabBarLength = 40;

	if (o == Qt::Vertical)
	{
		int bTop = y + ( h / 2) - 2;
		int halfWay = w / 2;
		int xStart = halfWay - (grabBarLength / 2);

		for (int k = 0; k < grabBarLength; k += 3)
		{
			p->drawRect(xStart + k, bTop, 1, 3);
		}
	}

	else if (o == Qt::Horizontal)
	{
		int bLeft = x + ( w / 2) - 2;
		int halfWay = h / 2;
		int yStart = halfWay - (grabBarLength / 2);

		for (int k = 0; k < grabBarLength; k += 3)
		{
			p->drawRect(bLeft, yStart + k, 3, 1);
		}
	}

	p->restore();
}

//as far as I can tell, this is never called.
void RENODeskStyle::drawRect( QPainter *p, int x, int y, int w, int h,
		       const QColor &c, int /*lineWidth*/, const QBrush *fill)
{
	p->save();
	p->setPen(c);
	p->setBrush(NoBrush);
	p->drawRect(x, y, w, h);

	if (fill)
	{
		p->fillRect(x + 1, y + 1, w - 2, h - 2, *fill);
	}

	p->restore();
}

//as far as I can tell, this is never called.
void RENODeskStyle::drawRectStrong ( QPainter *p, int x, int y, int w, int h,
			      const QColorGroup &g, bool /*sunken*/,
			      int /*lineWidth = 1*/, int /*midLineWidth*/,
			      const QBrush *fill)
{
	p->save();

	p->setPen(g.mid()); //I *don't* like strong rects -- don't agree? Here's the source
	p->setBrush(NoBrush);
	p->drawRect(x, y, w, h);

	if (fill)
	{
		p->fillRect(x + 1, y + 1, w - 2, h - 2, *fill);
	}

	p->restore();
}
