/*
 *  RENODeskStyle (C) 2002 Reinout Mechant  <reno@renoproductions.com>
 *
 *  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the Q Public License and the BSD license.
 *
 *  This library is free software; you should have received a copy of the
 *  Q Public License and of the BSD License along with this library;
 *  see the files COPYING.QPL and COPYING.BSD.
 *
 */


#ifndef __RENODeskStyle_h__
#define __RENODeskStyle_h__

#include <kstyle.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qpalette.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qbitmap.h>
#include <qintdict.h>
#include <qstrlist.h>
#include <qobject.h>
#include <limits.h>
#include <ksharedpixmap.h>
#include <krootpixmap.h>

class KPixmap;

#define MENU_CONFIG_CHANGED 667

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Enumerated Types
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

enum TransType { none=0, Custom };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : TransMenuHandler
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class TransMenuHandler : public QObject
{
	Q_OBJECT

public:
	TransMenuHandler(QObject *parent);
	~TransMenuHandler(){;}
	void reloadSettings();
	int transType(){return(type);}
	int menucolorType(){return(menucolortype);}
	KPixmap *pixmap(WId id){return(pixDict.find(id));}
	const QColor& menuColor(){return(menumenuColor);}

protected slots:
	void slotKIPCMessage(int id, int);

protected:
	bool eventFilter(QObject *obj, QEvent *ev);

	int menuOpacity;
	int type;
	int menucolortype;
	QColor menumenuColor;
	QIntDict<KPixmap>pixDict;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : RENODeskStyle
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class RENODeskStyle: public KStyle
{
friend class TransMenuHandler;
	public:

	RENODeskStyle();
	~RENODeskStyle();

	void polish(QApplication *);
	void polish(QWidget *);
	void polish(QPalette &);
	void unPolish(QApplication *);
	void unPolish(QWidget *);
	void polishPopupMenu(QPopupMenu *mnu);

	void drawButton(QPainter *p, int x, int y, int w, int h,
			const QColorGroup &g, bool sunken =
			false, const QBrush *fill = 0);

	QRect buttonRect(int x, int y, int w, int h);

	void drawBevelButton(QPainter *p, int x, int y, int w, int h,
			     const QColorGroup &g, bool sunken =
			     false, const QBrush *fill = 0);

	void drawPushButton(QPushButton *, QPainter *p);

	virtual void drawPushButtonLabel(QPushButton *, QPainter *p);

	void drawScrollButton(QPainter *p, int x, int y, int w, int h,
			      const QColorGroup &g, bool sunken,
			      const QBrush *fill);

	void drawScrollBarControls(QPainter *, const QScrollBar *,
				   int sliderStart, uint controls,
				   uint activeControl);

	QStyle::ScrollControl scrollBarPointOver(const QScrollBar *,
						  int sliderStart,
						  const QPoint &);

	void scrollBarMetrics(const QScrollBar *, int &sliderMin,
			      int &sliderMax, int &sliderLength,
			      int &buttonDim);

	QSize indicatorSize() const;

	void drawIndicator(QPainter *, int x, int y, int w, int h,
			   const QColorGroup &, int, bool down =
			   false, bool enabled = true);

	QSize exclusiveIndicatorSize() const;

	void drawExclusiveIndicator(QPainter *, int x, int y, int w, int h,
				    const QColorGroup &, bool on,
				    bool down = false, bool enabled =
				    true);

	void drawIndicatorMask(QPainter *, int x, int y, int w, int h,
			       int state);
	void drawExclusiveIndicatorMask(QPainter *, int x, int y, int w,
					int h, bool on);

	void drawComboButton(QPainter *, int x, int y, int w, int h,
			     const QColorGroup &, bool sunken =
			     false, bool editable = false, bool enabled =
			     true, const QBrush * fill = 0);

	QRect comboButtonRect(int x, int y, int w, int h);

	QRect comboButtonFocusRect(int x, int y, int w, int h);

	int sliderLength() const;

	void drawSliderGroove(QPainter *, int x, int y, int w, int h,
			      const QColorGroup &, QCOORD, Orientation);

	void drawArrow(QPainter *, Qt::ArrowType, bool down, int x, int y,
		       int w, int h, const QColorGroup &, bool enabled =
		       true, const QBrush * fill = 0);

	void drawSlider(QPainter *, int x, int y, int w, int h,
			const QColorGroup &, Orientation, bool tickAbove,
			bool tickBelow);

	void drawKBarHandle(QPainter *, int x, int y, int w, int h,
			    const QColorGroup &, KToolBarPos,
			    QBrush * fill = 0);

	void drawKMenuBar(QPainter *, int x, int y, int w, int h,
			  const QColorGroup &, bool macMode,
			  QBrush * fill = 0);
	void drawKMenuItem(QPainter *, int x, int y, int w, int h,
			   const QColorGroup &, bool active, QMenuItem *,
			   QBrush * fill = 0);

	void drawKToolBar(QPainter *, int x, int y, int w, int h,
			  const QColorGroup &, KToolBarPos, QBrush * fill =
			  0);

	void drawKToolBarButton(QPainter * p, int x, int y, int w, int h,
				const QColorGroup & g, bool sunken =
				false, bool raised = true, bool enabled =
				true, bool popup = false, KToolButtonType =
				Icon, const QString & btext =
				QString::null, const QPixmap * =
				0, QFont * = 0, QWidget * button = 0);

	void drawPopupMenuItem(QPainter *p, bool checkable, int maxpmw,
			       int tab, QMenuItem *mi, const QPalette &pal,
			       bool act, bool enabled, int x, int y, int w,
			       int h);

	int popupMenuItemHeight(bool c, QMenuItem *mi, const QFontMetrics &fm);

	void drawKProgressBlock(QPainter * p, int x, int y, int w, int h,
				const QColorGroup &, QBrush * fill);

	void drawFocusRect(QPainter * p, const QRect &, const QColorGroup &,
			   const QColor * pen, bool atBorder = false);

	void drawPanel(QPainter * p, int x, int y, int w, int h,
		       const QColorGroup &, bool sunken, int lineWidth =
		       1, const QBrush * = 0);

	void drawPopupPanel(QPainter *p, int x, int y, int w, int h,
			    const QColorGroup &g, int lineWidth =
			    2, const QBrush * = 0);

	void drawSeparator(QPainter * p, int x, int y, int w, int h,
			   const QColorGroup &, bool sunken =
			   true, int lineWidth = 1, int midLineWidth = 0);

	void drawTab(QPainter * p, const QTabBar * tabBar, QTab * tab,
		     bool selected);

	void drawTabMask(QPainter * p, const QTabBar *, QTab * tab, bool);

	void drawKickerHandle(QPainter * p, int x, int y, int w, int h,
			      const QColorGroup & g, QBrush *);

	void drawKickerAppletHandle(QPainter * p, int x, int y, int w,
				    int h, const QColorGroup & g,
				    QBrush *);

	void drawKickerTaskButton(QPainter * p, int x, int y, int w, int h,
				  const QColorGroup & g,
				  const QString & title, bool active,
				  QPixmap * icon, QBrush *);

	void drawSplitter ( QPainter *p, int x, int y, int w, int h,
			    const QColorGroup & g, Orientation o);

	void drawRect( QPainter *p, int x, int y, int w, int h,
		       const QColor &c, int lineWidth = 1, const QBrush *fill = 0);

	void drawRectStrong ( QPainter *p, int x, int y, int w, int h,
			      const QColorGroup &g, bool sunken = FALSE,
			      int lineWidth = 1, int midLineWidth = 0,
			      const QBrush *fill = 0);


	GUIStyle guiStyle() const
	{
		return Qt::MotifStyle;
	}

	bool eventFilter(QObject *, QEvent *);

      private:
    	TransMenuHandler *menuHandler;

};

#endif
